;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x1c262e7631d3fbc7db1402449c5ff362 . "./code.cg2js/cg-acache.fasl")
(#x70ef5e722deafa6cfc2aeedee0214fdb . "./code.cg2js/cg-base.fasl")
(#x5bd0951ec4ea8c97ac87d90593843287 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x5edfc42258c2d3a2f5e5442a44ef1de4 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x1e5c89bf0fa3b40a7e402513b01022dd . "./code.cg2js/cg-bitmap-stream.fasl")
(#xd36150021491ec0a1dcb3f19b36bd048 . "./code.cg2js/cg-button.fasl")
(#xf1588dad195d300b24fd25f557889469 . "./code.cg2js/cg-calendar.fasl")
(#xe87fef3d3c55a63ad35a26b3b8749a47 . "./code.cg2js/cg-caret.fasl")
(#xecdc1d50fd2322aa9d375147655f2028 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xa4e57472bc811dd73ce5d48ba81f83c9 . "./code.cg2js/cg-chart-widget.fasl")
(#x35f3ffeee3e10993165a7a625f7b824c . "./code.cg2js/cg-check-box.fasl")
(#xa7ad0f28cfe4ef8cc2307685e77d1ced . "./code.cg2js/cg-choice-list.fasl")
(#xb672b3df10f56f40ce804d077afbef64 . "./code.cg2js/cg-choose-printer.fasl")
(#x53fd32a957ab6d896c7368fed134328f . "./code.cg2js/cg-class-grid.fasl")
(#x0cee014e213acf30d12d5695476f1c5c . "./code.cg2js/cg-class-slot-grid.fasl")
(#x5d3cca2fe89997b74471411fe32b8509 . "./code.cg2js/cg-class-support.fasl")
(#xc488b2fd6bc852b756bf9ed7e3f0404e . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xebc770ed07c7018ee9f83048465f7ce2 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xa0f7bb2fefd0f9505138d4bbb0902695 . "./code.cg2js/cg-clipboard.fasl")
(#xf4a6c1430aa073ba5214582074c5cd6f . "./code.cg2js/cg-color-dialog.fasl")
(#xfbe3429f6c0f60f5856e528a831aabdb . "./code.cg2js/cg-combo-box.fasl")
(#x7a6f377c293ab23221005e90e4d5560d . "./code.cg2js/cg-common-control.fasl")
(#xbcdcc70621f9b9de7ca5ad81272daf46 . "./code.cg2js/cg-comtab.fasl")
(#xc75fffbfdecec9b04972ff0cc66f5b41 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xb67e095034b3830c3b9b75197486c4ac . "./code.cg2js/cg-curve.fasl")
(#xc090597605ee2cbddd0ae37a1e1d1987 . "./code.cg2js/cg-dde-utils.fasl")
(#xc1a5564bbb39da08e3e43f21a7504607 . "./code.cg2js/cg-dev.fasl")
(#x494570ca73483b23ab92d4f438d85364 . "./code.cg2js/cg-dialog-item.fasl")
(#xfa659bf191941543cf96155f1270aa2b . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xde09d09cb8835f569f8e5cf3198cb11f . "./code.cg2js/cg-directory-dialog.fasl")
(#x912dc378bc10a4ef2230d5dcba628691 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x55ceb6de68c3727e804133cdfdc38c7c . "./code.cg2js/cg-drag-and-drop.fasl")
(#x43db5c3f45549c6e0968f2afc9ce23d8 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x8aaeba48a2ed8344ebe4b36e34048200 . "./code.cg2js/cg-drawable.fasl")
(#xa87968ce325aa9e2c81d5e12a60b2d29 . "./code.cg2js/cg-dropping-outline.fasl")
(#x773f60035e57529e398786b95565fcc3 . "./code.cg2js/cg-edit-in-place.fasl")
(#x246c61c25a9a4c3fd442000dc6d35c2b . "./code.cg2js/cg-editable-text.fasl")
(#x30511d8b650368335bea76ee5150cc13 . "./code.cg2js/cg-file-dialog.fasl")
(#x8ecbb32960f9246f958878b14b699db8 . "./code.cg2js/cg-fill-texture.fasl")
(#x6790eaa756b4e300ce84cb5d459065d6 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x19185f496acaf4fb41694c9cd6eb30f2 . "./code.cg2js/cg-font-dialog.fasl")
(#xb5edee2e89f10afe78461c3c9f4169cb . "./code.cg2js/cg-gesture-emulation.fasl")
(#xee16614006b64ee9a665e924b981d790 . "./code.cg2js/cg-get-pixmap.fasl")
(#xe8f217c0c0c684563801623503cc6200 . "./code.cg2js/cg-get-position.fasl")
(#xef1beab2e809860bfa16b455128fd530 . "./code.cg2js/cg-graphics-context.fasl")
(#xcf2f93a0cd0940d7e94aa9275b46b11f . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xf33fc99035a97e0c971698a0600931dd . "./code.cg2js/cg-grid-widget.fasl")
(#x8f6fab998d37a17b3fee48344063ffdf . "./code.cg2js/cg-group-box.fasl")
(#x2b3017b582329f340750e8b4f5f8c35f . "./code.cg2js/cg-header-control.fasl")
(#x68437aced4355eff1e8b51b8972313aa . "./code.cg2js/cg-hotspot.fasl")
(#x6b7e1458c3ca0c693360b65051b3aaf2 . "./code.cg2js/cg-html-dialog.fasl")
(#xc818000a108f8c24eeae29c9d5e0f34b . "./code.cg2js/cg-html-widget.fasl")
(#x5c37c8dc190c401595ea4c093ab07706 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xf9eaab872adc2257af72a1d42d5f38d2 . "./code.cg2js/cg-icon.fasl")
(#x16d54ee760dd14e4ad49ac2a947db8ca . "./code.cg2js/cg-ie.fasl")
(#x1b48645c8e0b5935316440aec7bff4ac . "./code.cg2js/cg-intersect-posbox.fasl")
(#x15d1a175207f0305c435ea2213ef77d1 . "./code.cg2js/cg-intersect.fasl")
(#xd41a34c6abb3e821041f106562187d58 . "./code.cg2js/cg-item-list.fasl")
(#xb69d3ba7d0e8845eca3ec232d4179b80 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x40add9431d85b1a592938ddad382e6d2 . "./code.cg2js/cg-lamp.fasl")
(#x742218703f2eb6297c31cfb339652a46 . "./code.cg2js/cg-layout.fasl")
(#x65bd6f9a3f9ed0a508ed54598aea74c6 . "./code.cg2js/cg-lettered-menu.fasl")
(#x7e8954198005d0ecb4c7995333faa6d0 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x25613c67926426caf40d674ad2f7d363 . "./code.cg2js/cg-lisp-text.fasl")
(#x9b194ff4d59dc9608f248bc2ce95f520 . "./code.cg2js/cg-lisp-widget.fasl")
(#x162ab01ca31a4ef0cdc0391438a31c1c . "./code.cg2js/cg-list-view.fasl")
(#xb9eb8239dfd375071eb54fde58c1eef9 . "./code.cg2js/cg-mci.fasl")
(#xf90517d05ce2839561afe2e1392cdc6d . "./code.cg2js/cg-menu-tooltip.fasl")
(#xee32c5c06cf9ad7941fb84f3c13d7f8d . "./code.cg2js/cg-menu.fasl")
(#xe3dcc83a048c93a2b28da96dd1d2802c . "./code.cg2js/cg-message-dialog.fasl")
(#x24473d29730e9e529d831ff40f81c3e6 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xf9714552a7ebcce246b69a1d6af89916 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x35f3395fb721c82e65eee3265df2b381 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x33d6265eff02a1e071a79b1184fe7930 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x9ca522ad9966f7aa680f655187302d09 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x6ddfc88cd12566a7fb8c5c90cdaf0ac1 . "./code.cg2js/cg-nodes.fasl")
(#xbb844dc1707fda2375c774666e6417bb . "./code.cg2js/cg-object-editor-layout.fasl")
(#x6e392847dce565aafd552fa9837b3905 . "./code.cg2js/cg-object-editor.fasl")
(#x89bbc177d5ffe18f216eec3d85219b79 . "./code.cg2js/cg-ocx.fasl")
(#xdf9e854a50a1029618491e37433a7f67 . "./code.cg2js/cg-os-widget.fasl")
(#x9d45a206d27cc2411300740dacf13f45 . "./code.cg2js/cg-os-window.fasl")
(#x0c74b37fa9030b1c69b62e3f7e098e64 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x6fefa27ee852aefcddd6d94130950838 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x2f4713ba81a8c7fa7530c32a9b08cf34 . "./code.cg2js/cg-outline.fasl")
(#x8d7728498e1003b794b5e96249aa6612 . "./code.cg2js/cg-palette.fasl")
(#x883381c681e425a1d9b65258aa6c4ced . "./code.cg2js/cg-paren-matching.fasl")
(#x47c1a7a6e96bfd6f84b27bdb94c237a2 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x541655b65a66d1880b331daa31ef11b7 . "./code.cg2js/cg-picture-widget.fasl")
(#x82fa04278f3549e275f7d755fcc69f0b . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x448779b21b5d076e32ef734fc6c108ae . "./code.cg2js/cg-pixmap-printing.fasl")
(#x1e8feaf03f2aaee1dbe4e38a4df8de62 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xe8bafce84407670a1e28edba49f83a8b . "./code.cg2js/cg-pixmap-widget.fasl")
(#x3de83c88164be7c4fdd7272eef205db5 . "./code.cg2js/cg-pixmap.fasl")
(#x90a64b63c821916fc900fe8db36c1fc2 . "./code.cg2js/cg-plot-widget.fasl")
(#x920b7eac19bae28ed6df0ffb3165ef9c . "./code.cg2js/cg-printing.fasl")
(#xfcaa3f9a984e6cd48d3d3737bcbb8378 . "./code.cg2js/cg-progress-indicator.fasl")
(#xb80951262e2525a4997003782ae60b58 . "./code.cg2js/cg-project-window.fasl")
(#x27869b654ca78878edcf9824968063e7 . "./code.cg2js/cg-property.fasl")
(#xb72ef3e81d6569ea5201b46b47957202 . "./code.cg2js/cg-radio-button.fasl")
(#x06dabeda8b6b1c6fb977870bb51b7ea5 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x01039df8b1438d9c0a5d462f85d4f6fd . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xb69ca79cdc242439fa5017b1bfd9883c . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xa0ada88990dda48832473c1476cd65dd . "./code.cg2js/cg-rich-edit.fasl")
(#xdfa97b3b9be5392eab8a836e45676def . "./code.cg2js/cg-sample-file-menu.fasl")
(#xa65be3652732edba14835ec14a30b367 . "./code.cg2js/cg-scaling-stream.fasl")
(#xde555e42554bce73fec58a25bc447efe . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xd12aa29bd96a9f099bf9da2a44baaafd . "./code.cg2js/cg-scroll-bar.fasl")
(#xa8def3716858c7bd67e9bc8834728070 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x081743ac7203815a04327f1d2651a529 . "./code.cg2js/cg-selected-object.fasl")
(#x4879e75ed0eb8d1b65cc7e14f9ff6bab . "./code.cg2js/cg-shortcut-menu.fasl")
(#x3a4bc74c7166957159ec17eb6fa25f5c . "./code.cg2js/cg-split-bar.fasl")
(#x60708d02e81bd8e52164baa43528c405 . "./code.cg2js/cg-static-text.fasl")
(#xfb6f088bb28b3a0caeb8a39fd493ee88 . "./code.cg2js/cg-status-bar.fasl")
(#x90e6f7738b70a645ede6237068ed2ab2 . "./code.cg2js/cg-string-dialog.fasl")
(#x64fc0afa889e7b4ea1687aaf01e5faf1 . "./code.cg2js/cg-tab-control.fasl")
(#xe7f6e246e0c5d5ddb304da3dad28de74 . "./code.cg2js/cg-template-string.fasl")
(#x880495daf03504d9f6679568e3e6bc93 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xb2d482e15f9fd6cbb355dec9dd24321b . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x84abfa0ccef932d2324280b0a4d77ffc . "./code.cg2js/cg-text-edit-pane.fasl")
(#xda90e0e3ea22a7be7e2034551c2e75fb . "./code.cg2js/cg-text-or-combo.fasl")
(#x71ff7f4438cbfb39d897b8870e99f7dc . "./code.cg2js/cg-text-widget.fasl")
(#x5bf0ec0883a25dcd6a702776348f3544 . "./code.cg2js/cg-timer.fasl")
(#xfa45fd34f52fb7f3f8cd437b6a4cf3de . "./code.cg2js/cg-toggling-widget.fasl")
(#xd83bca8bf85ec5a28c890fa5f5c6da6e . "./code.cg2js/cg-toolbar.fasl")
(#x868f275ecb51a86b93b96ad6138599ee . "./code.cg2js/cg-tooltip.fasl")
(#xba9a0d4b1c6424fd33d8819642d7f1b8 . "./code.cg2js/cg-touch.fasl")
(#xe03fb9a6d2fd79c02386c61d26779775 . "./code.cg2js/cg-trackbar.fasl")
(#x08ce8f09359dcbda3cf8f7045d99f9b5 . "./code.cg2js/cg-tray.fasl")
(#xf99c48113894105b5390e566df0e375d . "./code.cg2js/cg-up-down-control.fasl")
(#xd79f46392cb8409c273fbf900dec9fc6 . "./code.cg2js/cg-utility-dialog.fasl")
(#xb788c3ea97909d573d70dae47819b104 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x1d2f5eb11f8f9f9eda1a9658f0ecd112 . "./code.cg2js/cg-web-browser.fasl")
(#x43798e2347728f3cf0ec2255bb6e02f5 . "./code.cg2js/cg-wrap-string.fasl")
(#x9315df9b5ca33d030702b79ea2026213 . "./code.cg2js/cg-yes-no-list.fasl")
(#xba4cde42e0c7694f459d0e6e323bd376 . "./code.cg2js/cg-yes-no-string.fasl")
(#x254794fb4d07c4256ee189a95bb69968 . "./code.cg2js/cg.fasl")
(#x7bea74c2e6031abfbb33c3e9c4390bd4 . "./code.idejs/ide-acache.fasl")
(#x9feafdea1457a2d9843d9964cce1f2f7 . "./code.idejs/ide-apropos.fasl")
(#x5372108ccd213727660240175290fe73 . "./code.idejs/ide-base.fasl")
(#x5ca6c021305cca24b45c54e396983e31 . "./code.idejs/ide-class-browser.fasl")
(#xe68d5424b6f210578e41023907caed26 . "./code.idejs/ide-class-grapher.fasl")
(#xc6cc46d844e653b85255171240480af3 . "./code.idejs/ide-clipboard.fasl")
(#x0aa6246e96c053542299e74ca69c8988 . "./code.idejs/ide-dev.fasl")
(#x13ee6a988d60eb10dcae0e6248966659 . "./code.idejs/ide-editor.fasl")
(#xd86d43948861865f6fb5a2c1ca7e3b4e . "./code.idejs/ide-find-definitions.fasl")
(#x0c7248d083a5f613de1072409d16e36b . "./code.idejs/ide-find-in-files.fasl")
(#xfed0fcf26f789aacb1e98eed528cf110 . "./code.idejs/ide-form.fasl")
(#x03ecc2fd6cf1cd6f219d517fade0ea48 . "./code.idejs/ide-grapher.fasl")
(#xf3ac50dc8e1f737b91da556a9cfad28a . "./code.idejs/ide-inspector.fasl")
(#x6e6eae9c8f8b3cfab9fe00273b364905 . "./code.idejs/ide-intro.fasl")
(#xc2bb83a6bc6a3e88e09cfecb196c6f6e . "./code.idejs/ide-keyshort.fasl")
(#x3b1c8bd261b08203e2c86c04a50b46ba . "./code.idejs/ide-navigator.fasl")
(#x976e6a1c515cf9a4d33ba765904fecdd . "./code.idejs/ide-package-list.fasl")
(#xacc1b73b2f99e004b84b24a31dab3384 . "./code.idejs/ide-patch-dialog.fasl")
(#x7e068f2e56f50feb80f36f19041dbece . "./code.idejs/ide-process-browser.fasl")
(#xa55f10839d82d7535eea5689b1bdeb46 . "./code.idejs/ide-project.fasl")
(#x4de130110587fa29101cbb2d40818852 . "./code.idejs/ide-trace.fasl")
(#xb0126e50d2fded2125d0ad9aa5a70279 . "./code.idejs/ide-window-list.fasl")
(#x2af4b68769a0a7eb834fd05c60751b69 . "./code.idejs/ide.fasl")
(#x82407180256cdbcae17acb711a3ed09f . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
